<?php

/*
 * @author     M2E Pro Developers Team
 * @copyright  M2E LTD
 * @license    Commercial use is forbidden
 */

/**
 * @method string tablename($tableName)
 * @method mixed insert($sql, $params = null)
 * @method \PDOStatement select($sql, $params = null)
 * @method void log($data, $type = 'std', $useprefix = true)
 *
 * Class M2eProChangesCatcher
 */
class M2eProChangesCatcher extends Magmi_ItemProcessor
{
    const INSTRUCTION_TYPE_PRODUCT_CHANGED = 'magmi_plugin_product_changed';
    const INSTRUCTION_INITIATOR            = 'public_services_magmi_plugin';

    protected $changedMagentoProductIds = [];
    protected $statistics = [
        'total_magento_products'      => 0,
        'total_listings_products'     => 0,
        'processed_listings_products' => 0
    ];

    //########################################

    /**
     * {@inheritdoc}
     */
    public function initialize($params) {}

    /**
     * {@inheritdoc}
     */
    public function getPluginInfo()
    {
        return [
            'name'    => 'Ess M2ePro Product Changes Inspector',
            'author'  => 'ESS',
            'version' => '2.2.0',
            'url'     => 'https://docs.m2epro.com/x/CQ1PAQ' // TODO CHANGE to M2 LINK IN FUTURE
        ];
    }

    //########################################

    /**
     * @param array $item
     * @param array|null $params
     * @return bool
     */
    public function processItemAfterId(&$item, $params = null)
    {
        if (empty($params['product_id'])) {
            return false;
        }

        $productId = (int) $params['product_id'];
        if (!in_array($productId, $this->changedMagentoProductIds, true)) {
            $this->changedMagentoProductIds[] = $productId;
        }

        return true;
    }

    /**
     * @return bool
     * @throws Exception
     */
    public function endImport()
    {
        $this->processChanges();

        $this->saveStatistics();
        $this->resetStatistics();

        return true;
    }

    //########################################

    private function processChanges()
    {
        $this->statistics['total_magento_products'] = count($this->changedMagentoProductIds);
        if (count($this->changedMagentoProductIds) == 0) {
            return;
        }

        $listingsProductsData = $this->getChangedListingsProductsData();
        $this->statistics['total_listings_products'] = count($listingsProductsData);

        if (empty($listingsProductsData)) {
            return;
        }

        $notProcessedListingsProductsData = $this->filterAlreadyProcessedListingsProducts($listingsProductsData);
        $this->statistics['processed_listings_products'] = count($notProcessedListingsProductsData);

        if (empty($notProcessedListingsProductsData)) {
            return;
        }

        $currentDateTime = new DateTime('now', new DateTimeZone('UTC'));
        $instructionTable = $this->tablename('m2epro_listing_product_instruction');

        $insertSql = "INSERT INTO `{$instructionTable}`
                      (`listing_product_id`,`component`,`type`,`initiator`,`priority`,`create_date`)
                      VALUES (?,?,?,?,?,?)";

        foreach ($notProcessedListingsProductsData as $listingProductData) {
            $instructionData = [
                'listing_product_id' => $listingProductData['listing_product_id'],
                'component'          => $listingProductData['component'],
                'type'               => self::INSTRUCTION_TYPE_PRODUCT_CHANGED,
                'initiator'          => self::INSTRUCTION_INITIATOR,
                'priority'           => 50,
                'create_date'        => $currentDateTime->format('Y-m-d H:i:s'),
            ];

            $this->insert($insertSql, array_values($instructionData));
        }
    }

    /**
     * @return array
     */
    private function getChangedListingsProductsData()
    {
        if (empty($this->changedMagentoProductIds)) {
            return [];
        }

        $listingProductTable  = $this->tablename('m2epro_listing_product');
        $variationTable       = $this->tablename('m2epro_listing_product_variation');
        $variationOptionTable = $this->tablename('m2epro_listing_product_variation_option');

        $listingsProductsData = [];

        foreach (array_chunk($this->changedMagentoProductIds, 1000) as $changedMagentoProductsIdsPart) {

            $productsIds = implode(',', $changedMagentoProductsIdsPart);
            $stmt = $this->select(<<<SQL
SELECT DISTINCT `id` AS `listing_product_id`, `component_mode`
FROM `{$listingProductTable}`
WHERE `product_id` IN ({$productsIds})

UNION

SELECT DISTINCT `lpv`.`listing_product_id`, `lpv`.`component_mode` FROM `{$variationOptionTable}` AS `lpvo`
INNER JOIN `{$variationTable}` AS `lpv` ON `lpv`.`id` = `lpvo`.`listing_product_variation_id`
WHERE `lpvo`.`product_id` IN ({$productsIds})
SQL
            );

            while ($row = $stmt->fetch()) {

                $listingProductId = (int)$row['listing_product_id'];

                $listingsProductsData[$listingProductId] = [
                    'listing_product_id' => $listingProductId,
                    'component'          => $row['component_mode'],
                ];
            }
        }

        return $listingsProductsData;
    }

    private function filterAlreadyProcessedListingsProducts(array $listingsProductsData)
    {
        $instructionTable = $this->tableName('m2epro_listing_product_instruction');

        foreach (array_chunk(array_keys($listingsProductsData), 1000) as $listingsProductsIdsPart) {

            $productsIds = implode(',', $listingsProductsIdsPart);
            $stmt = $this->select(<<<SQL
SELECT DISTINCT `listing_product_id` FROM `{$instructionTable}`
WHERE `listing_product_id` IN ($productsIds) AND `type` = ?
SQL
                , [self::INSTRUCTION_TYPE_PRODUCT_CHANGED]
            );

            while ($row = $stmt->fetch()) {
                $lpId = (int)$row['listing_product_id'];
                unset($listingsProductsData[$lpId]);
            }
        }

        return $listingsProductsData;
    }

    //########################################

    protected function saveStatistics()
    {
        $messages = [];

        $messages[] = "Total Magento Products: {$this->statistics['total_magento_products']}.";
        $messages[] = "Total Listings Products: {$this->statistics['total_listings_products']}.";
        $messages[] = "Processed Listings Products: {$this->statistics['processed_listings_products']}.";

        $this->log(implode(' ## ', $messages));
    }

    protected function resetStatistics()
    {
        $this->statistics = [
            'total_magento_products'      => 0,
            'total_listings_products'     => 0,
            'processed_listings_products' => 0
        ];
    }

    //########################################
}
